from __future__ import annotations

from dataclasses import asdict, dataclass
from enum import Enum
from typing import Any


class Shape(Enum):
    ALL_SHAPES = ""
    OUTLINE = "outline"
    FILL = "fill"
    LINEAL_COLOR = "lineal-color"
    HAND_DRAWN = "hand-drawn"


class OrderBy(Enum):
    POPULER = 4
    RECENT = 2


@dataclass
class Image:
    id: int
    name: str
    url: str

    pack: str
    downloads: int

    def to_dict(self) -> dict[str, Any]:
        return asdict(self)



from pytest import fixture, mark

from flaticon import FlatIconClient

pytestmark = mark.asyncio


@fixture
def client() -> FlatIconClient:
    return FlatIconClient()


async def test_download(client: FlatIconClient):
    images = await client.search_for_images("cat", count=2)
    image_paths = [await client.download_image(image) for image in images]
    assert image_paths



from pathlib import Path

from aiofiles import open as aioopen
from aiohttp import ClientSession


async def download(url: str, filepath: str | Path, session: ClientSession) -> Path:
    async with session.get(url) as response:
        async with aioopen(filepath, "wb") as file:
            async for chunk in response.content.iter_chunked(4096):
                await file.write(chunk)
    return Path(filepath)



"""Setup script for package

References: 
    - https://github.com/openai/openai-python/blob/main/setup.cfg
"""

from glob import glob
from os.path import basename, splitext
from setuptools import find_packages, setup
from pathlib import Path

VERSION = "1.0.0"
README_PATH = "docs/README.md"
DESCRIPTION = "Scrape icons from flaticon.com"
USERNAME = "yedhrab"
REPOSITORY = "flaticon"
KEYWORDS = ["flaticon", "icon", "scrape"]
INSTALL_REQUIRES = ["aiohttp==3.8.4", "beautifulsoup4==4.12.2", "aiofiles==23.1.0"]
EXSTRAS_REQUIRE = {
    "dev": [
        "black==23.3.0",
        "autoflake==2.0.2",
        "pytest==7.3.0",
        "pytest-asyncio==0.21.0",
    ]
}

setup(
    name=REPOSITORY,
    version=VERSION,
    license="Apache Software License 2.0",
    description=DESCRIPTION,
    long_description=Path(README_PATH).read_text(encoding="utf-8"),
    long_description_content_type="text/markdown",
    author="Yunus Emre Ak",
    author_email="yemreak.com@gmail.com",
    url=f"https://github.com/{USERNAME}/{REPOSITORY}",
    packages=find_packages(),
    py_modules=[splitext(basename(path))[0] for path in glob(f"*.py")],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Topic :: Utilities",
    ],
    project_urls={
        "Source": f"https://github.com/{USERNAME}/{REPOSITORY}/",
        "Changelog": f"https://github.com/{USERNAME}/{REPOSITORY}/releases",
        "Issue Tracker": f"https://github.com/{USERNAME}/{REPOSITORY}/issues",
    },
    keywords=KEYWORDS,
    python_requires=">=3.9.6",
    install_requires=INSTALL_REQUIRES,
    extras_require=EXSTRAS_REQUIRE,
    setup_requires=[],
    entry_points={},
)



from __future__ import annotations

from os import makedirs
from pathlib import Path
from typing import Any

from aiohttp import ClientSession
from bs4 import BeautifulSoup
from .utils import download

from .models import Image, OrderBy, Shape


class FlatIconClient:
    datapath: Path

    _session: ClientSession
    _csv_file: Path

    def __init__(self, relative_path: str = "flaticon") -> None:
        self.datapath = Path.home() / relative_path
        self._csv_file = self.datapath / "images.csv"

        makedirs(self.datapath, exist_ok=True)
        if not self._csv_file.exists():
            self._csv_file.touch()
            self._csv_file.write_text(",".join(Image.__annotations__))

    async def __aenter__(self):
        self._session = ClientSession()
        return self

    async def __aexit__(self, *args: Any):
        await self._session.close()

    async def search_for_images(
        self,
        query: str,
        shape: Shape = Shape.ALL_SHAPES,
        order_by: OrderBy = OrderBy.POPULER,
        count: int = 10,
    ) -> list[Image]:
        """Search query for flat icon

        Args:
            query (str): Image name
            shape (Shape, optional): Flat icon shapes. Defaults to "".
            order_by (OrderBy, optional): Order type. Defaults to 4.

        Returns:
            list[str]: Url of images
        """

        def create_url():
            url = f"https://www.flaticon.com/search?word={query}"
            if shape != Shape.ALL_SHAPES:
                url += f"&shape={shape.value}"
            url += f"&order_by={order_by}"
            return url

        images: list[Image] = []
        async with self._session.get(create_url()) as page:
            soup = BeautifulSoup(str(await page.content.read()), "html.parser")
            results = soup.find_all("li", class_="icon--item")
            for result in results[:count]:
                try:
                    image_url = Image(
                        id=result["data-png"]
                        .split("?")[0]
                        .split("/")[-1]
                        .split(".")[0],
                        name=result["data-name"],
                        url=result["data-png"],
                        downloads=result.get("data-downloads", 0),
                        pack=result.get("data-pack_name", "Unknown"),
                    )
                    images.append(image_url)
                except:  # Perevents any error related by ads
                    pass
        return images

    async def download_image(self, image: Image) -> Path:
        imagepath = self.datapath / f"{image.id}.png"
        with self._csv_file.open("r") as csv_file:
            for line in csv_file.readlines():
                if str(image.id) in line.split(",")[0]:
                    return imagepath
        await download(image.url, imagepath, self._session)
        with self._csv_file.open("a") as file:
            file.write("\n" + ",".join(image.to_dict().values()))
        return imagepath
